@@ -1,6 +1,7 @@
 #' @include backend-postgres.R
 NULL
 
+# nocov start
 # Use dbplyr edition 1 for custom method dispatch on RPostgreSQL connections
 #' @export
 dbplyr_edition.PostgreSQLConnection <- function(con) {
@@ -12,9 +13,9 @@ db_write_table.PostgreSQLConnection <- function(con, table, types, values,
                                                 temporary = TRUE, overwrite = FALSE, ....) {
 
   if (!isFALSE(temporary)) {
-    abort(c(
-      "RPostgreSQL backend does not support creation of temporary tables`",
-      i = "Either set `temporary = FALSE` or switch to RPostgres"
+    cli_abort(c(
+      "RPostgreSQL backend does not support creation of temporary tables",
+      i = "Either set {.code temporary = FALSE} or switch to {.pkg RPostgres}"
     ))
   }
 
@@ -55,3 +56,15 @@ sql_expr_matches.PostgreSQLConnection <- sql_expr_matches.PqConnection
 
 #' @export
 sql_query_explain.PostgreSQLConnection <- sql_query_explain.PqConnection
+
+#' @export
+supports_window_clause.PostgreSQLConnection <- function(con) {
+  TRUE
+}
+
+#' @export
+sql_query_insert.PostgreSQLConnection <- sql_query_insert.PostgreSQL
+
+#' @export
+sql_query_upsert.PostgreSQLConnection <- sql_query_upsert.PostgreSQL
+# nocov end
